@@ -17,7 +17,7 @@ import wx.html as wxhtml
 
 import numpy as np
 
-import fsl.utils.transform            as transform
+import fsl.transform.affine           as affine
 import fsl.utils.settings             as fslsettings
 import fsl.data.image                 as fslimage
 import fsl.data.mesh                  as fslmesh
@@ -30,6 +30,8 @@ import fsleyes_widgets.elistbox       as elistbox
 import fsleyes_widgets.utils.status   as status
 import fsleyes_widgets.utils.typedict as td
 
+import fsleyes.controls.controlpanel  as ctrlpanel
+import fsleyes.views.canvaspanel      as canvaspanel
 import fsleyes.panel                  as fslpanel
 import fsleyes.strings                as strings
 
@@ -37,7 +39,7 @@ import fsleyes.strings                as strings
 log = logging.getLogger(__name__)
 
 
-class LocationPanel(fslpanel.FSLeyesPanel):
+class LocationPanel(ctrlpanel.ControlPanel):
     """The ``LocationPanel`` is a panel which contains controls allowing the
     user to view and modify the :attr:`.DisplayContext.location` property.
 
@@ -62,11 +64,27 @@ class LocationPanel(fslpanel.FSLeyesPanel):
     """
 
 
+    @staticmethod
+    def supportedViews():
+        """The ``LocationPanel`` is restricted for use with
+        :class:`.CanvasPanel` views.
+        """
+        return [canvaspanel.CanvasPanel]
+
+
+    @staticmethod
+    def defaultLayout():
+        """Returns a dictionary containing layout settings to be passed to
+        :class:`.ViewPanel.togglePanel`.
+        """
+        return {'location' : wx.BOTTOM}
+
+
     def __init__(self,
                  parent,
                  overlayList,
                  displayCtx,
-                 frame,
+                 viewPanel,
                  showHistory=False):
         """Creat a ``LocationPanel``.
 
@@ -77,13 +95,13 @@ class LocationPanel(fslpanel.FSLeyesPanel):
 
         :arg displayCtx:  The :class:`.DisplayContext` instance.
 
-        :arg frame:       The :class:`.FSLeyesFrame` instance.
+        :arg viewPanel:   The :class:`.ViewPanel` instance.
 
         :arg showHistory: Defaults to ``False``. If ``True``, create and
                           display a :class:`LocationHistoryPanel`.
         """
-        fslpanel.FSLeyesPanel.__init__(
-            self, parent, overlayList, displayCtx, frame)
+        ctrlpanel.ControlPanel.__init__(
+            self, parent, overlayList, displayCtx, viewPanel)
 
         # only use a notebook if showing history panel
         if showHistory:
@@ -97,12 +115,12 @@ class LocationPanel(fslpanel.FSLeyesPanel):
 
         # We always create an info panel
         self.__info = LocationInfoPanel(
-            subparent, overlayList, displayCtx, frame)
+            subparent, overlayList, displayCtx, viewPanel.frame)
 
         # We don't always create a history panel
         if showHistory:
             self.__history = LocationHistoryPanel(
-                subparent, overlayList, displayCtx, frame, parent)
+                subparent, overlayList, displayCtx, viewPanel.frame, parent)
             self.__notebook.AddPage(self.__info,
                                     strings.labels[self, 'info'])
             self.__notebook.AddPage(self.__history,
@@ -131,7 +149,7 @@ class LocationPanel(fslpanel.FSLeyesPanel):
         self.__info    = None
         self.__history = None
 
-        fslpanel.FSLeyesPanel.destroy(self)
+        ctrlpanel.ControlPanel.destroy(self)
 
 
 class LocationInfoPanel(fslpanel.FSLeyesPanel):
@@ -407,7 +425,7 @@ class LocationInfoPanel(fslpanel.FSLeyesPanel):
             if w > width:  width  = w
             if h > height: height = h
 
-        return width + 5, height + 5
+        return width + 5, height
 
 
     def __selectedOverlayChanged(self, *a):
@@ -656,7 +674,7 @@ class LocationInfoPanel(fslpanel.FSLeyesPanel):
             shape    = refImage.shape[:3]
             vlo      = [0, 0, 0]
             vhi      = np.array(shape) - 1
-            wlo, whi = transform.axisBounds(shape, v2w)
+            wlo, whi = affine.axisBounds(shape, v2w)
             wstep    = refImage.pixdim[:3]
         else:
             vlo     = [0, 0, 0]
@@ -700,7 +718,7 @@ class LocationInfoPanel(fslpanel.FSLeyesPanel):
                   between the three location properties.
         """
 
-        if not self or self.destroyed():
+        if not self or self.destroyed:
             return
 
         if len(self.overlayList) == 0:       return
@@ -833,6 +851,11 @@ class LocationInfoPanel(fslpanel.FSLeyesPanel):
         selOvl     = displayCtx.getSelectedOverlay()
         lines      = []
 
+        dswarn = self.__genDisplaySpaceWarning()
+        if dswarn is not None:
+            fmt = '<span style="color: #ff0000"><b>{}</b></span>'
+            lines.append(fmt.format(dswarn))
+
         for overlay in overlays:
 
             display = displayCtx.getDisplay(overlay)
@@ -848,59 +871,9 @@ class LocationInfoPanel(fslpanel.FSLeyesPanel):
             # corresponds to a vertex, show some info
             # about that vertex
             if isinstance(overlay, fslmesh.Mesh):
-                vidx  = opts.getVertex()
-                vd    = opts.getVertexData()
-                vdidx = opts.vertexDataIndex
-
-                if vidx is None:
-                    info = '[no vertex]'
-
-                else:
-                    # some vertex data has been
-                    # loaded for this mesh.
-                    if vd is not None:
-
-                        # time series/multiple data points per
-                        # vertex - display the time/data index
-                        # as well
-                        if vd.shape[1] > 1:
-                            info = '[{}, {}]: {}'.format(vidx,
-                                                         vdidx,
-                                                         vd[vidx, vdidx])
-
-                        # Only one scalar value per vertex -
-                        # don't bother showing the vertex
-                        # data index
-                        else:
-                            info = '[{}]: {}'.format(vidx, vd[vidx, vdidx])
-
-                    else:
-                        info = '[{}]'.format(vidx)
-
+                info = self.__genMeshInfo(overlay, opts)
             elif isinstance(overlay, fslimage.Image):
-
-                vloc = opts.getVoxel()
-
-                if vloc is not None:
-                    vloc = tuple(int(v) for v in vloc)
-                    vloc = opts.index(vloc)
-                    vval = overlay[vloc]
-                    vloc = ' '.join(map(str, vloc))
-
-                    if not np.isscalar(vval):
-                        vval = np.asscalar(vval)
-
-                    if opts.overlayType == 'label':
-                        lbl = opts.lut.get(int(vval))
-                        if lbl is None: lbl = 'no label'
-                        else:           lbl = lbl.name
-                        info = '[{}]: {} / {}'.format(vloc, vval, lbl)
-
-                    else:
-                        info = '[{}]: {}'.format(vloc, vval)
-
-                else:
-                    info = strings.labels[self, 'outOfBounds']
+                info = self.__genImageInfo(overlay, opts)
             else:
                 info = '{}'.format(strings.labels[self, 'noData'])
 
@@ -922,6 +895,88 @@ class LocationInfoPanel(fslpanel.FSLeyesPanel):
         self.__info.Refresh()
 
 
+    def __genDisplaySpaceWarning(self):
+        """Generate a warning if images with different orientations and/or
+        fields-of-view are loaded.
+        """
+        images  = [o for o in self.overlayList
+                   if isinstance(o, fslimage.Image)]
+
+        for i in images[1:]:
+            if not i.sameSpace(images[0]):
+                return strings.messages[self, 'displaySpaceWarning']
+
+        return None
+
+
+    def __genMeshInfo(self, ovl, opts):
+        """Generate an info line for the given :class:`.Mesh` overlay. """
+        vidx  = opts.getVertex()
+        vd    = opts.getVertexData()
+        vdidx = opts.vertexDataIndex
+
+        if vidx is None:
+            info = '[no vertex]'
+
+        else:
+            # some vertex data has been
+            # loaded for this mesh.
+            if vd is not None:
+
+                # time series/multiple data points per
+                # vertex - display the time/data index
+                # as well
+                if vd.shape[1] > 1:
+                    info = '[{}, {}]: {}'.format(vidx,
+                                                 vdidx,
+                                                 vd[vidx, vdidx])
+
+                # Only one scalar value per vertex -
+                # don't bother showing the vertex
+                # data index
+                else:
+                    info = '[{}]: {}'.format(vidx, vd[vidx, vdidx])
+
+            else:
+                info = '[{}]'.format(vidx)
+        return info
+
+
+    def __genImageInfo(self, ovl, opts):
+        """Generate an info line for the given :class:`.Image` overlay. """
+
+        vloc = opts.getVoxel()
+
+        if vloc is not None:
+            vloc = tuple(int(v) for v in vloc)
+            vloc = opts.index(vloc)
+            vval = ovl[vloc]
+            vloc = ' '.join(map(str, vloc))
+
+            if not np.isscalar(vval):
+                vval = np.asscalar(vval)
+
+            if opts.overlayType == 'label':
+                lbl = opts.lut.get(int(vval))
+                if lbl is None: lbl = 'no label'
+                else:           lbl = lbl.name
+                info = '[{}]: {} / {}'.format(vloc, vval, lbl)
+
+            elif opts.overlayType == 'complex':
+                if opts.component in ('real', 'imag'):
+                    info = '[{}]: {}'.format(vloc, vval)
+                else:
+                    cval = opts.getComponent(vval)
+                    info = '[{}]: {} ({}: {})'.format(
+                        vloc, vval, opts.component, cval)
+            else:
+                info = '[{}]: {}'.format(vloc, vval)
+
+        else:
+            info = strings.labels[self, 'outOfBounds']
+        return info
+
+
 DISPLAYOPTS_BOUNDS = td.TypeDict({
     'DisplayOpts' : ['bounds'],
     'MeshOpts'    : ['refImage'],
@@ -935,8 +990,9 @@ listeners on the properties specified in this dictionary.
 
 
 DISPLAYOPTS_INFO = td.TypeDict({
-    'NiftiOpts' : ['volume'],
-    'MeshOpts'  : ['vertexDataIndex'],
+    'NiftiOpts'   : ['volume'],
+    'MeshOpts'    : ['vertexDataIndex'],
+    'ComplexOpts' : ['component']
 })
 """Different :class:`.DisplayOpts` types have different properties which
 affect the current overlay location information.  Therefore, when the current
@@ -1023,7 +1079,9 @@ class LocationHistoryPanel(fslpanel.FSLeyesPanel):
         self.__save .Bind(wx.EVT_BUTTON,                 self.__onSave)
         self.__clear.Bind(wx.EVT_BUTTON,                 self.__onClear)
 
-        self.__canvas.addListener('profile', self.name, self.__profileChanged)
+        self.__canvas.events.register(self.name,
+                                      self.__profileChanged,
+                                      'profile')
         self.__profileChanged()
 
 
@@ -1032,7 +1090,7 @@ class LocationHistoryPanel(fslpanel.FSLeyesPanel):
         needed.
         """
         self.__deregisterProfile()
-        self.__canvas.removeListener('profile', self.name)
+        self.__canvas.events.deregister(self.name, 'profile')
         self.__canvas = None
         fslpanel.FSLeyesPanel.destroy(self)
 
@@ -1076,7 +1134,7 @@ class LocationHistoryPanel(fslpanel.FSLeyesPanel):
         mouse event listeners with the new :class:`.Profile` object.
         """
         self.__deregisterProfile()
-        self.__registerProfile(self.__canvas.getCurrentProfile())
+        self.__registerProfile(self.__canvas.currentProfile)
 
 
     def __addLocation(self, worldLoc, comment=None):
