@@ -13,7 +13,6 @@ import logging
 import copy
 
 import fsleyes_props      as props
-import fsleyes.colourmaps as fslcm
 
 from . import sceneopts
 from . import canvasopts
@@ -57,10 +56,6 @@ class OrthoOpts(sceneopts.SceneOpts):
     """
 
 
-    labelSize = props.Int(minval=4, maxval=96, default=14, clamped=True)
-    """Label font size."""
-
-
     layout = props.Choice(('horizontal', 'vertical', 'grid'))
     """How should we lay out each of the three canvases?"""
 
@@ -77,6 +72,30 @@ class OrthoOpts(sceneopts.SceneOpts):
     """Controls zoom on the Z canvas. """
 
 
+    invertXHorizontal = copy.copy(canvasopts.SliceCanvasOpts.invertX)
+    """Invert the X canvas along the horizontal axis. """
+
+
+    invertXVertical = copy.copy(canvasopts.SliceCanvasOpts.invertY)
+    """Invert the X canvas along the vertical axis. """
+
+
+    invertYHorizontal = copy.copy(canvasopts.SliceCanvasOpts.invertX)
+    """Invert the Y canvas along the horizontal axis. """
+
+
+    invertYVertical = copy.copy(canvasopts.SliceCanvasOpts.invertY)
+    """Invert the Y canvas along the vertical axis. """
+
+
+    invertZHorizontal = copy.copy(canvasopts.SliceCanvasOpts.invertX)
+    """Invert the Z canvas along the horizontal axis. """
+
+
+    invertZVertical = copy.copy(canvasopts.SliceCanvasOpts.invertY)
+    """Invert the Z canvas along the vertical axis. """
+
+
     def __init__(self, *args, **kwargs):
         """Create an ``OrthoOpts`` instance. All arguments are passed
         through to the :class:`.SceneOpts` constructor.
