@@ -8,13 +8,12 @@
 use with :class:`.DisplayOpts` classes.
 """
 
-import deprecation
-
 import numpy as np
 
-from   fsl.utils.platform import platform as fslplatform
-import fsl.utils.transform                as transform
-import fsleyes_props                      as props
+import fsl.transform.affine as affine
+import fsleyes_props        as props
+import fsleyes_widgets      as fwidgets
+import fsleyes.gl           as fslgl
 
 
 class Volume3DOpts(object):
@@ -35,6 +34,12 @@ class Volume3DOpts(object):
     """
 
 
+    blendByIntensity  = props.Boolean(default=True)
+    """If ``True``, the colours from samples are weighted by voxel intensity
+    as well as the blendFactor.
+    """
+
+
     numSteps = props.Int(minval=25, maxval=500, default=100, clamped=False)
     """Specifies the maximum number of samples to acquire in the rendering of
     each pixel of the 3D scene. This corresponds to the number of iterations
@@ -126,7 +131,7 @@ class Volume3DOpts(object):
         # If we're in an X11/SSh session,
         # step down the quality so it's
         # a bit faster.
-        if fslplatform.inSSHSession:
+        if fwidgets.inSSHSession():
             self.numSteps    = 60
             self.resolution  = 70
             self.blendFactor = 0.3
@@ -135,7 +140,7 @@ class Volume3DOpts(object):
         # maximum possible amount of
         # smoothing, as GL14 fragment
         # programs cannot be too large.
-        if float(fslplatform.glVersion) < 2.1:
+        if float(fslgl.GL_COMPATIBILITY) < 2.1:
             smooth = self.getProp('smoothing')
             smooth.setAttribute(self, 'maxval', 6)
 
@@ -156,15 +161,6 @@ class Volume3DOpts(object):
         pass
 
 
-    @property
-    @deprecation.deprecated(deprecated_in='0.17.0',
-                            removed_in='1.0.0',
-                            details='Dithering is automatically calculated')
-    def dithering(self):
-        """Deprecated."""
-        pass
-
-
     def getNumSteps(self):
         """Return the value of the :attr:`numSteps` property, possibly
         adjusted according to the the :attr:`numInnerSteps` property. The
@@ -174,7 +170,7 @@ class Volume3DOpts(object):
         See the :class:`.GLVolume` class for more details.
         """
 
-        if float(fslplatform.glVersion) >= 2.1:
+        if float(fslgl.GL_COMPATIBILITY) >= 2.1:
             return self.numSteps
 
         outer = self.getNumOuterSteps()
@@ -236,15 +232,15 @@ class Volume3DOpts(object):
         # it by the inverse modelview
         # matrix
         t2dmat = self.getTransform('texture', 'display')
-        xform  = transform.concat(view, t2dmat)
-        ixform = transform.invert(xform)
+        xform  = affine.concat(view, t2dmat)
+        ixform = affine.invert(xform)
 
-        eye    = transform.transform(eye,    ixform, vector=True)
-        target = transform.transform(target, ixform, vector=True)
+        eye    = affine.transform(eye,    ixform, vector=True)
+        target = affine.transform(target, ixform, vector=True)
 
         # Direction that the 'camera' is
         # pointing, normalied to unit length
-        cdir = transform.normalise(eye - target)
+        cdir = affine.normalise(eye - target)
 
         # Calculate the length of one step
         # along the camera direction in a
@@ -265,8 +261,8 @@ class Volume3DOpts(object):
         #
         # The projection matrix puts depth into
         # [-1, 1], but we want it in [0, 1]
-        zscale = transform.scaleOffsetXform([1, 1, 0.5], [0, 0, 0.5])
-        xform  = transform.concat(zscale, proj, xform)
+        zscale = affine.scaleOffsetXform([1, 1, 0.5], [0, 0, 0.5])
+        xform  = affine.concat(zscale, proj, xform)
 
         return rayStep, xform
 
@@ -297,12 +293,12 @@ class Volume3DOpts(object):
         centre = [xmid, ymid, zmid]
         normal = [0, 0, -1]
 
-        rot1     = transform.axisAnglesToRotMat(incline, 0, 0)
-        rot2     = transform.axisAnglesToRotMat(0, 0, azimuth)
-        rotation = transform.concat(rot2, rot1)
+        rot1     = affine.axisAnglesToRotMat(incline, 0, 0)
+        rot2     = affine.axisAnglesToRotMat(0, 0, azimuth)
+        rotation = affine.concat(rot2, rot1)
 
-        normal = transform.transformNormal(normal, rotation)
-        normal = transform.normalise(normal)
+        normal = affine.transformNormal(normal, rotation)
+        normal = affine.normalise(normal)
 
         offset = (pos - 0.5) * max((b.xlen, b.ylen, b.zlen))
         origin = centre + normal * offset
