@@ -6,6 +6,11 @@
 #
 """This module provides the entry point to *FSLeyes*, the FSL image viewer.
 
+Call the :func:`main` function to start the main FSLeyes application.
+
+The :func:`embed` function can be called to open a :class:`.FSLeyesFrame`
+within an existing application.
+
 See the :mod:`fsleyes` package documentation for more details on ``fsleyes``.
 
 
@@ -23,25 +28,32 @@ See the :mod:`fsleyes` package documentation for more details on ``fsleyes``.
 """
 
 
-import            os
-import os.path as op
-import            sys
-import            signal
-import            logging
-import            textwrap
+import functools as ft
+import os.path   as op
+import              os
+import              sys
+import              signal
+import              logging
+import              textwrap
 
 import wx
-import appnope
+import wx.adv
 
-from fsl.utils.platform import platform as fslplatform
+from   fsl.utils.platform import platform as fslplatform
+import fsl.utils.idle                     as idle
+import fsleyes_widgets                    as fwidgets
+import fsleyes_widgets.utils.status       as status
 
 import                       fsleyes
+import fsleyes.strings    as strings
 import fsleyes.splash     as fslsplash
+import fsleyes.cliserver  as cliserver
 import fsleyes.colourmaps as colourmaps
 
 
 # wx.ModalDialogHook does not exist in wxPython < 4
-if fslplatform.wxFlavour == fslplatform.WX_PYTHON:
+
+if fwidgets.wxFlavour() in (fwidgets.WX_PYTHON, fwidgets.WX_UNKNOWN):
     class ModalDialogHook(object):
         def Register(self):
             pass
@@ -74,7 +86,7 @@ class FSLeyesApp(wx.App):
             return wx.ID_NONE
 
         def Exit(self, dlg):
-            self.modals.remove(dlg)
+            self.modals.discard(dlg)
 
 
     def __init__(self):
@@ -83,6 +95,15 @@ class FSLeyesApp(wx.App):
         self.__overlayList = None
         self.__displayCtx  = None
 
+        # On macOS, when the user drags a file onto the FSLeyes window,
+        # or onto the FSLeyes.app icon, the file path will be passed to
+        # MacOpenFiles method. But that method may be called very early
+        # on in the startup process, before the DisplayContext and
+        # OverlayList have been created. So when this happens, we cache
+        # the files here, and then open them when the
+        # SetOverlayListAndDisplayContext method gets called.
+        self.__filesToOpen = []
+
         self.__modalHook = FSLeyesApp.ModalHook()
         self.__modalHook.Register()
 
@@ -90,6 +111,13 @@ class FSLeyesApp(wx.App):
 
         self.SetAppName('FSLeyes')
 
+        try:
+            self.__icon = wx.adv.TaskBarIcon(iconType=wx.adv.TBI_DOCK)
+            self.__icon.SetIcon(wx.Icon(
+                op.join(fsleyes.assetDir, 'icons', 'app_icon.png')))
+        except Exception:
+            self.__icon = None
+
 
     @property
     def modals(self):
@@ -105,6 +133,14 @@ class FSLeyesApp(wx.App):
         self.__overlayList = overlayList
         self.__displayCtx  = displayCtx
 
+        # MacOpenFiles was called before the
+        # overlaylist/dc were created, and
+        # queued some files that need to be
+        # opened
+        if len(self.__filesToOpen) > 0:
+            wx.CallAfter(self.MacOpenFiles, self.__filesToOpen)
+            self.__filesToOpen = None
+
 
     def MacReopenApp(self):
         """On OSX, make sure that the FSLeyes frame is restored if it is
@@ -149,13 +185,17 @@ class FSLeyesApp(wx.App):
         on the application icon.
         """
 
+        # OverlayList has not yet been created -
+        # queue the files to open them later
+        # in SetOverlayListAndDisplayContext
         if self.__overlayList is None:
+            self.__filesToOpen.extend(filenames)
             return
 
         import fsleyes.actions.loadoverlay as loadoverlay
         import fsleyes.autodisplay         as autodisplay
 
-        def onLoad(overlays):
+        def onLoad(paths, overlays):
 
             if len(overlays) == 0:
                 return
@@ -190,23 +230,18 @@ def main(args=None):
         render.main(args[1:])
         sys.exit(0)
 
-    # Hook which allows us to run a jupyter
-    # notebook server  within a frozen version
-    # of FSLeyes
-    if len(args) == 2 and args[0] == 'notebook' and fslplatform.frozen:
-        from notebook.notebookapp import main
-
-        # second argument is a path
-        # to add to the PYTHONPATH.
-        # See fsleyes.actions.notebook.NotebookServer.run.
-        sys.path.insert(0, args[1])
-        fsleyes.configLogging()
-        sys.exit(main(argv=[]))
-
     # the fsleyes.initialise function figures
     # out the path to asset files (e.g. cmaps)
     fsleyes.initialise()
 
+    # Hook which allows us to run a jupyter
+    # notebook server from an existing FSLeyes
+    # instance
+    if len(args) >= 1 and args[0] == 'notebook':
+        from fsleyes.actions.notebook import nbmain
+        fsleyes.configLogging()
+        sys.exit(nbmain(args))
+
     # initialise colour maps - this must be
     # done before parsing arguments, as if
     # the user asks for help, available
@@ -226,10 +261,12 @@ def main(args=None):
         return app, splash
 
     # If it looks like the user is asking for
-    # help, then we parse command line arguments
-    # before creating a wx.App and showing the
-    # splash screen. This means that FSLeyes
-    # help/version information can be retrieved
+    # help, or using cliserver to pass arguments
+    # to an existing FSLeyes instance, then we
+    # parse command line arguments before
+    # creating a wx.App and showing the splash
+    # screen. This means that FSLeyes help/
+    # version information can be retrieved
     # without a display, and hopefully fairly
     # quickly.
     #
@@ -251,9 +288,11 @@ def main(args=None):
     if (len(args) > 0) and (args[0] in ('-V',
                                         '-h',
                                         '-fh',
+                                        '-cs',
                                         '--version',
                                         '--help',
-                                        '--fullhelp')):
+                                        '--fullhelp',
+                                        '--cliserver')):
         namespace   = [parseArgs(args)]
         app, splash = initgui()
 
@@ -288,7 +327,11 @@ def main(args=None):
         # module will raise SystemExit.
         try:
             if namespace[0] is None:
-                namespace[0] = parseArgs(args)
+
+                errmsg   = strings.messages['main.parseArgs.error']
+                errtitle = strings.titles[  'main.parseArgs.error']
+                with status.reportIfError(errtitle, errmsg, raiseError=True):
+                    namespace[0] = parseArgs(args)
 
         # But the wx.App.MainLoop eats SystemExit
         # exceptions for unknown reasons, and
@@ -297,9 +340,9 @@ def main(args=None):
         # (e.g. code coverage) impossible. So I'm
         # catching SystemExit here, and then
         # telling the wx.App to exit gracefully.
-        except SystemExit as e:
+        except (SystemExit, Exception) as e:
             app.ExitMainLoop()
-            exitCode[0] = e.code
+            exitCode[0] = getattr(e, 'code', 1)
             return
 
         # Configure logging (this has to be done
@@ -320,7 +363,11 @@ def main(args=None):
         # and then create the FSLeyesFrame.
         overlayList, displayCtx = makeDisplayContext(namespace[0], splash)
         app.SetOverlayListAndDisplayContext(overlayList, displayCtx)
-        frame = makeFrame(namespace[0], displayCtx, overlayList, splash)
+        frame = makeFrame(namespace[0],
+                          displayCtx,
+                          overlayList,
+                          splash,
+                          [shutdown])
 
         app.SetTopWindow(frame)
         frame.Show()
@@ -340,6 +387,18 @@ def main(args=None):
                          showErrorMessage=False,
                          ignorePoint=False)
 
+        # start notebook server
+        if namespace[0].notebookFile is not None:
+            namespace[0].notebook     = True
+            namespace[0].notebookFile = op.abspath(namespace[0].notebookFile)
+        if namespace[0].notebook:
+            from fsleyes.actions.notebook import NotebookAction
+            frame.menuActions[NotebookAction](namespace[0].notebookFile)
+
+        # start CLI server
+        if namespace[0].cliserver:
+            cliserver.runserver(overlayList, displayCtx)
+
     # Shut down cleanly on sigint/sigterm.
     # We do this so that any functions
     # registered with atexit will actually
@@ -375,12 +434,104 @@ def main(args=None):
     # already created the splash screen, so
     # all is well.
     wx.CallAfter(init, splash)
-    appnope.nope()
+
+    # under mac, use appnope to make sure
+    # we don't get put to sleep. This is
+    # primarily for the jupyter notebook
+    # integration - if the user is working
+    # with a notebook in the web browser,
+    # macos might put FSLeyes to sleep,
+    # causing the kernel to become
+    # unresponsive.
+    try:
+        import appnope
+        appnope.nope()
+    except ImportError:
+        pass
     app.MainLoop()
-    shutdown()
     return exitCode[0]
 
 
+def embed(mkFrame=True, **kwargs):
+    """Initialise FSLeyes and create a :class:`.FSLeyesFrame`, when
+    running within another application.
+
+    .. note:: In most cases, this function must be called from the
+              ``wx.MainLoop``.
+
+    :arg mkFrame: Defaults to ``True``. If ``False``, FSLeyes is
+                  initialised, but a :class:`.FSLeyesFrame` is not created.
+                  If you set this to ``False``, you must ensure that a
+                  ``wx.App`` object exists before calling this function.
+
+    :returns:     A tuple containing:
+
+                   - The :class:`.OverlayList`
+                   - The master :class:`.DisplayContext`
+                   - The :class:`.FSLeyesFrame` (or ``None``, if
+                     ``makeFrame is False``).
+
+    All other arguments are passed to :meth:`.FSLeyesFrame.__init__`.
+    """
+
+    import fsleyes_props          as props
+    import fsleyes.gl             as fslgl
+    import fsleyes.frame          as fslframe
+    import fsleyes.overlay        as fsloverlay
+    import fsleyes.displaycontext as fsldc
+
+    # initialise must be called before
+    # a FSLeyesApp gets created, as it
+    # tries to access app_icon.png
+    fsleyes.initialise()
+
+    app    = wx.GetApp()
+    ownapp = app is None
+
+    if ownapp and (mkFrame is False):
+        raise RuntimeError('If mkFrame is False, you '
+                           'must create a wx.App before '
+                           'calling fsleyes.main.embed')
+    if ownapp:
+        app = FSLeyesApp()
+
+    colourmaps.init()
+    props.initGUI()
+
+    called = [False]
+    ret    = [None]
+
+    def until():
+        return called[0]
+
+    def ready():
+        fslgl.bootstrap()
+
+        overlayList = fsloverlay.OverlayList()
+        displayCtx  = fsldc.DisplayContext(overlayList)
+
+        if mkFrame:
+            frame = fslframe.FSLeyesFrame(
+                None, overlayList, displayCtx, **kwargs)
+        else:
+            frame = None
+
+        if ownapp:
+            app.SetOverlayListAndDisplayContext(overlayList, displayCtx)
+            # Keep a ref to prevent the app from being GC'd
+            frame._embed_app = app
+
+        called[0] = True
+        ret[0]    = (overlayList, displayCtx, frame)
+
+    fslgl.getGLContext(ready=ready, raiseErrors=True)
+    idle.block(10, until=until)
+
+    if ret[0] is None:
+        raise RuntimeError('Failed to start FSLeyes')
+    return ret[0]
+
+
 def initialise(splash, namespace, callback):
     """Called by :func:`main`. Bootstraps/Initialises various parts of
     *FSLeyes*.
@@ -406,36 +557,6 @@ def initialise(splash, namespace, callback):
     # to the current working directory.
     curDir = op.normpath(os.getcwd())
 
-    # But if we are running as a frozen application, check to
-    # see if FSLeyes has been started by the system (e.g.
-    # double-clicking instead of being called from the CLI).
-    #
-    # If so, we set the save/load directory
-    # to the user's home directory instead.
-    if fslplatform.frozen:
-
-        fsleyesDir = op.dirname(__file__)
-
-        # If we're a frozen OSX application,
-        # we need to adjust the FSLeyes dir
-        # (which will be:
-        #   [install_dir]/FSLeyes.app/Contents/MacOS/fsleyes/),
-        #
-        # Because the cwd will default to:
-        #   [install_dir]/
-
-        if fslplatform.os == 'Darwin':
-
-            fsleyesDir = op.normpath(op.join(fsleyesDir,
-                                             '..', '..', '..', '..'))
-
-        # Similar adjustment for linux
-        elif fslplatform.os == 'Linux':
-            fsleyesDir = op.normpath(op.join(fsleyesDir, '..'))
-
-        if curDir == fsleyesDir:
-            curDir = op.expanduser('~')
-
     fslsettings.write('loadSaveOverlayDir', curDir)
 
     # Initialise silly things
@@ -455,9 +576,7 @@ def initialise(splash, namespace, callback):
     try:
         # Force the creation of a wx.glcanvas.GLContext object,
         # and initialise OpenGL version-specific module loads.
-        # The splash screen is used as the parent of the dummy
-        # canvas created by the gl.getWXGLContext function.
-        fslgl.getGLContext(parent=splash, ready=realCallback)
+        fslgl.getGLContext(ready=realCallback)
 
     except Exception:
         log.error('Unable to initialise OpenGL!', exc_info=True)
@@ -465,18 +584,28 @@ def initialise(splash, namespace, callback):
         sys.exit(1)
 
 
-def shutdown():
+def shutdown(ev=None):
     """Called when FSLeyes exits normally (i.e. the user closes the window).
     Does some final clean-up before exiting.
+
+    This function is used as a wxpython event handler, so it accepts an ``ev``
+    arguments, but ignores its value.
     """
 
+    if ev is not None:
+        ev.Skip()
+
     import fsl.utils.settings as fslsettings
+    import fsleyes.gl         as fslgl
 
     # Clear the cached directory for loading/saving
     # files - when FSLeyes starts up, we want it to
     # default to the current directory.
     fslsettings.delete('loadSaveOverlayDir')
 
+    # Shut down the GL rendering context
+    fslgl.shutdown()
+
 
 def parseArgs(argv):
     """Parses the given ``fsleyes`` command line arguments. See the
@@ -494,9 +623,15 @@ def parseArgs(argv):
         add_help=False,
         formatter_class=parseargs.FSLeyesHelpFormatter)
 
+    serveraction = ft.partial(cliserver.CLIServerAction, allArgs=argv)
+
     parser.add_argument('-r', '--runscript',
                         metavar='SCRIPTFILE',
                         help='Run custom FSLeyes script')
+    parser.add_argument('-cs', '--cliserver',
+                        action=serveraction,
+                        help='Pass all command-line arguments '
+                             'to a single FSLeyes instance')
 
     # We include the list of available
     # layouts in the help description
@@ -581,7 +716,7 @@ def makeDisplayContext(namespace, splash):
     return overlayList, displayCtx
 
 
-def makeFrame(namespace, displayCtx, overlayList, splash):
+def makeFrame(namespace, displayCtx, overlayList, splash, closeHandlers):
     """Creates the *FSLeyes* interface.
 
     This function does the following:
@@ -594,34 +729,32 @@ def makeFrame(namespace, displayCtx, overlayList, splash):
      3. Destroys the splash screen that was created by the :func:`context`
         function.
 
-    :arg namespace:   Parsed command line arguments, as returned by
-                      :func:`parseArgs`.
+    :arg namespace:     Parsed command line arguments, as returned by
+                        :func:`parseArgs`.
 
-    :arg displayCtx:  The  :class:`.DisplayContext`, as created and returned
-                      by :func:`makeDisplayContext`.
+    :arg displayCtx:    The  :class:`.DisplayContext`, as created and returned
+                        by :func:`makeDisplayContext`.
 
-    :arg overlayList: The :class:`.OverlayList`, as created and returned by
-                      :func:`makeDisplayContext`.
+    :arg overlayList:   The :class:`.OverlayList`, as created and returned by
+                        :func:`makeDisplayContext`.
 
-    :arg splash:      The :class:`.FSLeyesSplash` frame.
+    :arg splash:        The :class:`.FSLeyesSplash` frame.
+
+    :arg closeHandlers: List of event handlers to be called when the
+                        ``FSLeyesFrame`` closes.
 
     :returns: the :class:`.FSLeyesFrame` that was created.
     """
 
-    import fsl.utils.idle               as idle
-    import fsleyes_widgets.utils.status as status
-    import fsleyes.parseargs            as parseargs
-
-    # The fsleyes.actions.frameactions module
-    # monkey-patches some things into the
-    # FSLeyesFrame class, so it must be
-    # imported immediately after fsleyes.frame.
-    import fsleyes.frame                as fsleyesframe
-    import fsleyes.actions.frameactions as frameactions  # noqa
-
-    import fsleyes.displaycontext       as fsldisplay
-    import fsleyes.layouts              as layouts
-    import fsleyes.views.canvaspanel    as canvaspanel
+    import fsl.utils.idle                        as idle
+    import fsleyes_widgets.utils.status          as status
+    import fsleyes.parseargs                     as parseargs
+    import fsleyes.frame                         as fsleyesframe
+    import fsleyes.displaycontext                as fsldisplay
+    import fsleyes.layouts                       as layouts
+    import fsleyes.views.canvaspanel             as canvaspanel
+    import fsleyes.views.orthopanel              as orthopanel
+    import fsleyes.plugins.tools.saveannotations as saveannotations
 
     # Set up the frame scene (a.k.a. layout)
     # The scene argument can be:
@@ -648,7 +781,13 @@ def makeFrame(namespace, displayCtx, overlayList, splash):
         displayCtx,
         restore,
         True,
-        fontSize=namespace.fontSize)
+        fontSize=namespace.fontSize,
+        closeHandlers=closeHandlers)
+
+    # Allow files to be dropped
+    # onto FSLeyes to open them
+    dt = fsleyesframe.OverlayDropTarget(overlayList, displayCtx)
+    frame.SetDropTarget(dt)
 
     # Make sure the new frame is shown
     # before destroying the splash screen
@@ -704,6 +843,18 @@ def makeFrame(namespace, displayCtx, overlayList, splash):
         parseargs.applySceneArgs(
             namespace, overlayList, displayCtx, viewOpts)
 
+    # If an annotations file has eben specified,
+    # and an ortho view was opened, load the
+    # annotations file, and apply it to the
+    # first ortho view
+    orthos = [vp for vp in viewPanels if isinstance(vp, orthopanel.OrthoPanel)]
+    if namespace.annotations is not None and len(orthos) > 0:
+        try:
+            saveannotations.loadAnnotations(orthos[0], namespace.annotations)
+        except Exception as e:
+            log.warning('Error loading annotations from %s: %s',
+                        namespace.annotations, e, exc_info=True)
+
     # If a script has been specified, we run
     # the script. This has to be done on the
     # idle loop, because overlays specified
@@ -712,8 +863,9 @@ def makeFrame(namespace, displayCtx, overlayList, splash):
     # script on idle (which is a queue), the
     # script can assume that all overlays have
     # already been loaded.
+    from fsleyes.actions.runscript import RunScriptAction
     if script is not None:
-        idle.idle(frame.runScript, script)
+        idle.idle(frame.menuActions[RunScriptAction], script)
 
     return frame
 
