@@ -14,8 +14,10 @@ import            sys
 import            logging
 import            textwrap
 
+import numpy as np
+
+import fsl.utils.idle                        as idle
 import fsleyes_widgets.utils.layout          as fsllayout
-import fsleyes_widgets.utils.colourbarbitmap as cbarbitmap
 
 import                                          fsleyes
 import fsleyes.version                       as version
@@ -26,22 +28,20 @@ import fsleyes.displaycontext                as displaycontext
 import fsleyes.displaycontext.orthoopts      as orthoopts
 import fsleyes.displaycontext.lightboxopts   as lightboxopts
 import fsleyes.displaycontext.scene3dopts    as scene3dopts
+import fsleyes.controls.colourbar            as cbar
+import fsleyes.plugins.tools.saveannotations as saveannotations
 import fsleyes.gl                            as fslgl
+import fsleyes.gl.textures.imagetexture      as imagetexture
 import fsleyes.gl.ortholabels                as ortholabels
 import fsleyes.gl.offscreenslicecanvas       as slicecanvas
 import fsleyes.gl.offscreenlightboxcanvas    as lightboxcanvas
 import fsleyes.gl.offscreenscene3dcanvas     as scene3dcanvas
 
 
-
 log = logging.getLogger(__name__)
 
 
-CBAR_SIZE  = 75
-"""Height/width, in pixels, of a colour bar. """
-
-
-def main(args=None):
+def main(args=None, hook=None):
     """Entry point for ``render``.
 
     Creates and renders an OpenGL scene, and saves it to a file, according
@@ -52,31 +52,58 @@ def main(args=None):
     if args is None:
         args = sys.argv[1:]
 
-    # Create a GL context
-    fslgl.getGLContext(offscreen=True, createApp=True)
-
-    # Initialise FSLeyes and implement hacks
+    # Initialise FSLeyes and implement
+    # hacks. This must come first as it
+    # does a number of important things.
     fsleyes.initialise()
 
     # Initialise colour maps module
     fslcm.init()
 
-    # Parse arguments, and
-    # configure logging/debugging
-    namespace = parseArgs(args)
-    fsleyes.configLogging(namespace.verbose, namespace.noisy)
+    # Create a GL context
+    fslgl.getGLContext(offscreen=True, createApp=True)
 
-    # Initialise the fsleyes.gl modules
-    fslgl.bootstrap(namespace.glversion)
+    # Now that GL inititalisation is over,
+    # make sure that the idle loop executes
+    # all tasks synchronously, instead of
+    # trying to schedule them on the wx
+    # event loop. And make sure image textures
+    # don't use separate threads for data
+    # processing.
+    with idle.idleLoop.synchronous(), \
+         imagetexture.ImageTexture.enableThreading(False):
 
-    # Create a description of the scene
-    overlayList, displayCtx, sceneOpts = makeDisplayContext(namespace)
+        # Parse arguments, and
+        # configure logging/debugging
+        namespace = parseArgs(args)
+        fsleyes.configLogging(namespace.verbose, namespace.noisy)
 
-    import matplotlib.image as mplimg
+        # Initialise the fsleyes.gl modules
+        fslgl.bootstrap(namespace.glversion)
 
-    # Render that scene, and save it to file
-    bitmap = render(namespace, overlayList, displayCtx, sceneOpts)
-    mplimg.imsave(namespace.outfile, bitmap)
+        # Create a description of the scene
+        overlayList, displayCtx, sceneOpts = makeDisplayContext(namespace)
+
+        import matplotlib.image as mplimg
+
+        # Render that scene, and save it to file
+        bitmap, bg = render(
+            namespace, overlayList, displayCtx, sceneOpts, hook)
+
+        if namespace.crop is not None:
+            bitmap = autocrop(bitmap, bg, namespace.crop)
+
+        # Alpha-blending does work, but the final
+        # pixel values seem to take on the alpha
+        # value of the most recently drawn item,
+        # which is undesirable. So we save out
+        # as rgb
+        bitmap = bitmap[:, :, :3]
+
+        mplimg.imsave(namespace.outfile, bitmap)
+
+        # Clear the GL context
+        fslgl.shutdown()
 
 
 def parseArgs(argv):
@@ -94,7 +121,13 @@ def parseArgs(argv):
 
     mainParser.add_argument('-of',
                             '--outfile',
-                            help='Output image file name'),
+                            help='Output image file name')
+    mainParser.add_argument('-c',
+                            '--crop',
+                            type=int,
+                            metavar='BORDER',
+                            help='Auto-crop image, leaving a '
+                                 'border on each side')
     mainParser.add_argument('-sz',
                             '--size',
                             type=int, nargs=2,
@@ -112,17 +145,17 @@ def parseArgs(argv):
         ('ortho'), lightbox ('lightbox'), or 3D ('3d') views.
         """)
 
-    namespace = parseargs.parseArgs(mainParser,
-                                    argv,
-                                    name,
-                                    prolog=prolog,
-                                    desc=description,
-                                    usageProlog=optStr,
-                                    argOpts=['-of',
-                                             '--outfile',
-                                             '-sz',
-                                             '--size'],
-                                    shortHelpExtra=['--outfile', '--size'])
+    namespace = parseargs.parseArgs(
+        mainParser,
+        argv,
+        name,
+        prolog=prolog,
+        desc=description,
+        usageProlog=optStr,
+        argOpts=['-of', '--outfile',
+                 '-sz', '--size',
+                 '-c',  '--crop'],
+        shortHelpExtra=['--outfile', '--size', '--crop'])
 
     if namespace.outfile is None:
         log.error('outfile is required')
@@ -172,7 +205,8 @@ def makeDisplayContext(namespace):
         log.info('Loading overlay {} ...'.format(ovl))
 
     def error(ovl, error):
-        log.info('Error loading overlay {}: '.format(ovl, error))
+        log.error('Error loading overlay {}: {}'.format(ovl, error))
+        raise error
 
     # Load the overlays specified on the command
     # line, and configure their display properties
@@ -198,7 +232,7 @@ def makeDisplayContext(namespace):
     elif namespace.scene == 'lightbox':
         sceneOpts = lightboxopts.LightBoxOpts(MockCanvasPanel(1))
     elif namespace.scene == '3d':
-        sceneOpts = scene3dopts. Scene3DOpts(MockCanvasPanel(1))
+        sceneOpts = scene3dopts.Scene3DOpts(MockCanvasPanel(1))
 
     # 3D views default to
     # world display space
@@ -234,15 +268,22 @@ def makeDisplayContext(namespace):
     return overlayList, childDisplayCtx, sceneOpts
 
 
-def render(namespace, overlayList, displayCtx, sceneOpts):
-    """Renders the scene, and returns a bitmap.
+def render(namespace, overlayList, displayCtx, sceneOpts, hook=None):
+    """Renders the scene, and returns a tuple containing the bitmap and the
+    background colour.
 
     :arg namespace:   ``argparse.Namespace`` object containing command line
                       arguments.
-
     :arg overlayList: The :class:`.OverlayList` instance.
     :arg displayCtx:  The :class:`.DisplayContext` instance.
     :arg sceneOpts:   The :class:`.SceneOpts` instance.
+    :arg hook:        Function which is called after the canvases have been
+                      created, but before the scene is rendered. Can be used
+                      to perform any configuration/drawing in addition to that
+                      specified via ``namespace``.
+
+    .. note:: The ``hook`` argument was added for testing purposes, but may
+              be useful in other situations.
     """
 
     # Calculate canvas and colour bar sizes
@@ -253,7 +294,8 @@ def render(namespace, overlayList, displayCtx, sceneOpts):
         adjustSizeForColourBar(width,
                                height,
                                sceneOpts.showColourBar,
-                               sceneOpts.colourBarLocation)
+                               sceneOpts.colourBarLocation,
+                               sceneOpts.labelSize)
 
     # Lightbox view -> only one canvas
     if namespace.scene == 'lightbox':
@@ -317,15 +359,42 @@ def render(namespace, overlayList, displayCtx, sceneOpts):
     if namespace.scene == 'ortho' and sceneOpts.layout == 'grid':
         canvases[1].opts.invertX = True
 
+    # Load annotations, only on ortho
+    if namespace.scene == 'ortho' and namespace.annotations is not None:
+        saveannotations.loadAnnotations(MockOrthoPanel(canvases),
+                                        namespace.annotations)
+
     # Configure each of the canvases (with those
     # properties that are common to both ortho and
     # lightbox canvases) and render them one by one
     canvasBmps = []
 
-    for i, c in enumerate(canvases):
+    # Call hook if provided (used for testing)
+    if hook is not None:
+        hook(overlayList, displayCtx, sceneOpts, canvases)
+
+    for c in canvases:
 
         c.opts.pos = displayCtx.location
 
+        # HACK If a SliceCanvas/LightBoxCanvas
+        # is rendering the sceen to an off-screen
+        # texture due to the low performance
+        # setting, its internal viewport will not
+        # be set until after all GLObjects have
+        # been rendered. But some GLObjects (e.g.
+        # GLLabel) need to know the current
+        # viewport.
+        #
+        # This is very much an edge case, as who
+        # would be using a low performance setting
+        # for off-screen rendering?
+
+        if namespace.scene in ('ortho', 'lightbox') and \
+           namespace.performance is not None        and \
+           int(namespace.performance) < 3:
+            c._setViewport()
+
         c.draw()
 
         canvasBmps.append(c.getBitmap())
@@ -353,10 +422,7 @@ def render(namespace, overlayList, displayCtx, sceneOpts):
                                        displayCtx,
                                        cbarWidth,
                                        cbarHeight,
-                                       sceneOpts.colourBarLocation,
-                                       sceneOpts.colourBarLabelSide,
-                                       sceneOpts.bgColour,
-                                       sceneOpts.fgColour)
+                                       sceneOpts)
         if cbarBmp is not None:
             layout  = buildColourBarLayout(layout,
                                            cbarBmp,
@@ -364,8 +430,8 @@ def render(namespace, overlayList, displayCtx, sceneOpts):
                                            sceneOpts.colourBarLabelSide)
 
     # Turn the layout tree into a bitmap image
-    return fsllayout.layoutToBitmap(
-        layout, [c * 255 for c in sceneOpts.bgColour])
+    bgColour = [c * 255 for c in sceneOpts.bgColour]
+    return fsllayout.layoutToBitmap(layout, bgColour), bgColour
 
 
 def createLightBoxCanvas(namespace,
@@ -434,18 +500,25 @@ def createOrthoCanvases(namespace,
     canvasAxes = []
     zooms      = []
     centres    = []
+    inverts    = []
     if sceneOpts.showXCanvas:
         canvasAxes.append((1, 2))
         zooms     .append(sceneOpts.xzoom)
         centres   .append(sceneOpts.panel.getGLCanvases()[0].centre)
+        inverts   .append((sceneOpts.invertXHorizontal,
+                           sceneOpts.invertXVertical))
     if sceneOpts.showYCanvas:
         canvasAxes.append((0, 2))
         zooms     .append(sceneOpts.yzoom)
         centres   .append(sceneOpts.panel.getGLCanvases()[1].centre)
+        inverts   .append((sceneOpts.invertYHorizontal,
+                           sceneOpts.invertYVertical))
     if sceneOpts.showZCanvas:
         canvasAxes.append((0, 1))
         zooms     .append(sceneOpts.zzoom)
         centres   .append(sceneOpts.panel.getGLCanvases()[2].centre)
+        inverts   .append((sceneOpts.invertZHorizontal,
+                           sceneOpts.invertZVertical))
 
     # Grid layout only makes sense if
     # we're displaying 3 canvases
@@ -456,6 +529,7 @@ def createOrthoCanvases(namespace,
         canvasAxes = [canvasAxes[1], canvasAxes[0], canvasAxes[2]]
         centres    = [centres[   1], centres[   0], centres[   2]]
         zooms      = [zooms[     1], zooms[     0], zooms[     2]]
+        inverts    = [inverts[   1], inverts[   0], inverts[   2]]
 
     # Calculate the size in pixels for each canvas
     sizes = calculateOrthoCanvasSizes(overlayList,
@@ -466,10 +540,15 @@ def createOrthoCanvases(namespace,
                                       sceneOpts.layout)
 
     # Configure the properties on each canvas
-    for ((width, height), (xax, yax), zoom, centre) in zip(sizes,
-                                                           canvasAxes,
-                                                           zooms,
-                                                           centres):
+    for ((width, height),
+         (xax, yax),
+         zoom,
+         centre,
+         (invertx, inverty)) in zip(sizes,
+                                    canvasAxes,
+                                    zooms,
+                                    centres,
+                                    inverts):
 
         zax = 3 - xax - yax
 
@@ -486,6 +565,8 @@ def createOrthoCanvases(namespace,
         opts.cursorGap    = sceneOpts.cursorGap
         opts.bgColour     = sceneOpts.bgColour
         opts.renderMode   = sceneOpts.renderMode
+        opts.invertX      = invertx
+        opts.invertY      = inverty
 
         if zoom is not None:
             opts.zoom = zoom
@@ -499,7 +580,6 @@ def createOrthoCanvases(namespace,
 
         canvases.append(c)
 
-
     return canvases
 
 
@@ -537,8 +617,10 @@ def create3DCanvas(namespace,
     opts.offset          = sceneOpts.offset
     opts.rotation        = sceneOpts.rotation
 
-    if parseargs.wasSpecified(namespace, sceneOpts, 'lightPos'):
+    if parseargs.wasSpecified(namespace, sceneOpts, 'lightPos') or \
+       parseargs.wasSpecified(namespace, sceneOpts, 'lightDistance'):
         opts.lightPos        = sceneOpts.lightPos
+        opts.lightDistance   = sceneOpts.lightDistance
         canvas.resetLightPos = False
     else:
         canvas.defaultLightPos()
@@ -550,10 +632,7 @@ def buildColourBarBitmap(overlayList,
                          displayCtx,
                          width,
                          height,
-                         cbarLocation,
-                         cbarLabelSide,
-                         bgColour,
-                         fgColour):
+                         sceneOpts):
     """If the currently selected overlay has a display range,
     creates and returns a bitmap containing a colour bar. Returns
     ``None`` otherwise.
@@ -566,64 +645,34 @@ def buildColourBarBitmap(overlayList,
 
     :arg height:        Colour bar height in pixels.
 
-    :arg cbarLocation:  One of  ``'top'``, ``'bottom'``, ``'left'``, or
-                        ``'right'``.
-
-    :arg cbarLabelSide: One of ``'top-left'`` or ``'bottom-right'``.
-
-    :arg bgColour:      RGBA background colour.
-
-    :arg fgColour:      RGBA foreground (text) colour.
+    :arg sceneOpts:     :class:`.SceneOpts` instance containing display
+                        settings.
     """
 
     overlay = displayCtx.getSelectedOverlay()
     display = displayCtx.getDisplay(overlay)
     opts    = display.opts
 
+    cbarLocation = sceneOpts.colourBarLocation
+    cbarSize     = sceneOpts.colourBarSize
+
+    if   cbarLocation in ('top', 'bottom'): width  = width  * cbarSize / 100.0
+    elif cbarLocation in ('left', 'right'): height = height * cbarSize / 100.0
+
     if not isinstance(opts, displaycontext.ColourMapOpts):
         return None
 
     if   cbarLocation in ('top', 'bottom'): orient = 'horizontal'
     elif cbarLocation in ('left', 'right'): orient = 'vertical'
 
-    if   cbarLabelSide == 'top-left':
-        if orient == 'horizontal': labelSide = 'top'
-        else:                      labelSide = 'left'
-    elif cbarLabelSide == 'bottom-right':
-        if orient == 'horizontal': labelSide = 'bottom'
-        else:                      labelSide = 'right'
+    cb = cbar.ColourBar(overlayList, displayCtx)
+    cb.orientation = orient
+    cb.labelSide   = sceneOpts.colourBarLabelSide
+    cb.bgColour    = sceneOpts.bgColour
+    cb.textColour  = sceneOpts.fgColour
+    cb.fontSize    = sceneOpts.labelSize
 
-
-    if opts.useNegativeCmap:
-        negCmap    = opts.negativeCmap
-        ticks      = [0.0, 0.49, 0.51, 1.0]
-        ticklabels = ['{:0.2f}'.format(-opts.displayRange.xhi),
-                      '{:0.2f}'.format(-opts.displayRange.xlo),
-                      '{:0.2f}'.format( opts.displayRange.xlo),
-                      '{:0.2f}'.format( opts.displayRange.xhi)]
-        tickalign  = ['left', 'right', 'left', 'right']
-    else:
-        negCmap    = None
-        ticks      = [0.0, 1.0]
-        tickalign  = ['left', 'right']
-        ticklabels = ['{:0.2f}'.format(opts.displayRange.xlo),
-                      '{:0.2f}'.format(opts.displayRange.xhi)]
-
-    cbarBmp = cbarbitmap.colourBarBitmap(
-        cmap=opts.cmap,
-        width=width,
-        height=height,
-        negCmap=negCmap,
-        invert=opts.invert,
-        ticks=ticks,
-        ticklabels=ticklabels,
-        tickalign=tickalign,
-        label=display.name,
-        orientation=orient,
-        labelside=labelSide,
-        bgColour=bgColour,
-        textColour=fgColour,
-        cmapResolution=opts.cmapResolution)
+    cbarBmp = cb.colourBar(width, height)
 
     # The colourBarBitmap function returns a w*h*4
     # array, but the fsleyes_widgets.utils.layout.Bitmap
@@ -660,7 +709,11 @@ def buildColourBarLayout(canvasLayout,
     elif cbarLocation in ('left', 'right'): return fsllayout.HBox(items)
 
 
-def adjustSizeForColourBar(width, height, showColourBar, colourBarLocation):
+def adjustSizeForColourBar(width,
+                           height,
+                           showColourBar,
+                           colourBarLocation,
+                           fontSize):
     """Calculates the widths and heights of the image display space, and the
     colour bar if it is enabled.
 
@@ -674,6 +727,8 @@ def adjustSizeForColourBar(width, height, showColourBar, colourBarLocation):
     :arg colourBarLocation: Colour bar location (see
                             :func:`buildColourBarBitmap`).
 
+    :arg fontSize           Font size (points) used in colour bar labels.
+
     :returns:               Two tuples - the first tuple contains the
                             ``(width, height)`` of the available canvas space,
                             and the second contains the ``(width, height)`` of
@@ -682,7 +737,7 @@ def adjustSizeForColourBar(width, height, showColourBar, colourBarLocation):
 
     if showColourBar:
 
-        cbarWidth = CBAR_SIZE
+        cbarWidth = int(round(cbar.colourBarMinorAxisSize(fontSize)))
         if colourBarLocation in ('top', 'bottom'):
             height     = height - cbarWidth
             cbarHeight = cbarWidth
@@ -741,8 +796,43 @@ def calculateOrthoCanvasSizes(overlayList,
                                height)
 
 
+def autocrop(data, bgColour, border=0):
+    """Crops the given bitmap image on all sides where the ``bgColour`` is
+    the only colour present.
 
-class MockSliceCanvas(object):
+    If the image is completely empty. it is not cropped.
+
+    :arg data:     ``numpy`` array of shape ``(w, h, 4)`` containing the image.
+    :arg bgColour: Sequence of length 4 containing the background colour to
+                   crop.
+    :arg border:   Number of pixels to leave around each side.
+    """
+
+    w, h = data.shape[:2]
+
+    low, hiw = 0, w
+    loh, hih = 0, h
+
+    while np.all(data[low,     :] == bgColour): low += 1
+    while np.all(data[hiw - 1, :] == bgColour): hiw -= 1
+    while np.all(data[:, loh]     == bgColour): loh += 1
+    while np.all(data[:, hih - 1] == bgColour): hih -= 1
+
+    if low < hiw and loh < hih:
+        data = data[low:hiw, loh:hih, :]
+
+        if border > 0:
+            w, h, c = data.shape
+            new = np.zeros((w + 2 * border, h + 2 * border, c),
+                           dtype=data.dtype)
+            new[:, :] = bgColour
+            new[border:-border, border:-border, :] = data
+            data = new
+
+    return data
+
+
+class MockSliceCanvas:
     """Used in place of a :class:`.SliceCanvas`. The :mod:`.parseargs` module
     needs access to ``SliceCanvas`` instances to apply some command line
     options. However, ``render`` calls :func:`.parseargs.applySceneArgs`
@@ -762,7 +852,7 @@ class MockSliceCanvas(object):
         self.centre = x, y
 
 
-class MockCanvasPanel(object):
+class MockCanvasPanel:
     """Used in place of a :class:`.CanvasPanel`. This is used as a container
     for :class:`MockSliceCanvas` instances.
     """
@@ -772,5 +862,21 @@ class MockCanvasPanel(object):
         return self.canvases
 
 
+class MockOrthoPanel:
+    """Used in place of an :class:`.OrthoPanel`. This is used as a container
+    for three :class:`SliceCanvas` instances.
+    """
+    def __init__(self, canvases):
+        self.canvases = canvases
+    def getGLCanvases(self):
+        return self.canvases
+    def getXCanvas(self):
+        return self.canvases[0]
+    def getYCanvas(self):
+        return self.canvases[1]
+    def getZCanvas(self):
+        return self.canvases[2]
+
+
 if __name__ == '__main__':
     main()
